@@ -26,7 +26,7 @@ export const defaultMaxStatements = 1e9;
 
 
 // $function: arrayCopy
-// $group: Array
+// $group: array
 // $doc: Create a copy of an array
 // $arg array: The array to copy
 // $return: The array copy
@@ -41,7 +41,7 @@ const arrayCopyArgs = valueArgsModel([
 
 
 // $function: arrayDelete
-// $group: Array
+// $group: array
 // $doc: Delete an array element
 // $arg array: The array
 // $arg index: The index of the element to delete
@@ -61,7 +61,7 @@ const arrayDeleteArgs = valueArgsModel([
 
 
 // $function: arrayExtend
-// $group: Array
+// $group: array
 // $doc: Extend one array with another
 // $arg array: The array to extend
 // $arg array2: The array to extend with
@@ -78,8 +78,25 @@ const arrayExtendArgs = valueArgsModel([
 ]);
 
 
+// $function: arrayFlat
+// $group: array
+// $doc: Flat an array hierarchy
+// $arg array: The array to flat
+// $arg depth: The maximum depth of the array hierarchy
+// $return: The flated array
+function arrayFlat(args) {
+    const [array, depth] = valueArgsValidate(arrayFlatArgs, args);
+    return array.flat(depth);
+}
+
+const arrayFlatArgs = valueArgsModel([
+    {'name': 'array', 'type': 'array'},
+    {'name': 'depth', 'type': 'number', 'integer': true, 'default': 10}
+]);
+
+
 // $function: arrayGet
-// $group: Array
+// $group: array
 // $doc: Get an array element
 // $arg array: The array
 // $arg index: The array element's index
@@ -100,7 +117,7 @@ const arrayGetArgs = valueArgsModel([
 
 
 // $function: arrayIndexOf
-// $group: Array
+// $group: array
 // $doc: Find the index of a value in an array
 // $arg array: The array
 // $arg value: The value to find in the array, or a match function, f(value) -> bool
@@ -133,7 +150,7 @@ const arrayIndexOfArgs = valueArgsModel([
 
 
 // $function: arrayJoin
-// $group: Array
+// $group: array
 // $doc: Join an array with a separator string
 // $arg array: The array
 // $arg separator: The separator string
@@ -150,7 +167,7 @@ const arrayJoinArgs = valueArgsModel([
 
 
 // $function: arrayLastIndexOf
-// $group: Array
+// $group: array
 // $doc: Find the last index of a value in an array
 // $arg array: The array
 // $arg value: The value to find in the array, or a match function, f(value) -> bool
@@ -187,7 +204,7 @@ const arrayLastIndexOfArgs = valueArgsModel([
 
 
 // $function: arrayLength
-// $group: Array
+// $group: array
 // $doc: Get the length of an array
 // $arg array: The array
 // $return: The array's length; zero if not an array
@@ -202,7 +219,7 @@ const arrayLengthArgs = valueArgsModel([
 
 
 // $function: arrayNew
-// $group: Array
+// $group: array
 // $doc: Create a new array
 // $arg values...: The new array's values
 // $return: The new array
@@ -212,7 +229,7 @@ function arrayNew(values) {
 
 
 // $function: arrayNewSize
-// $group: Array
+// $group: array
 // $doc: Create a new array of a specific size
 // $arg size: Optional (default is 0). The new array's size.
 // $arg value: Optional (default is 0). The value with which to fill the new array.
@@ -229,7 +246,7 @@ const arrayNewSizeArgs = valueArgsModel([
 
 
 // $function: arrayPop
-// $group: Array
+// $group: array
 // $doc: Remove the last element of the array and return it
 // $arg array: The array
 // $return: The last element of the array; null if the array is empty.
@@ -247,7 +264,7 @@ const arrayPopArgs = valueArgsModel([
 
 
 // $function: arrayPush
-// $group: Array
+// $group: array
 // $doc: Add one or more values to the end of the array
 // $arg array: The array
 // $arg values...: The values to add to the end of the array
@@ -265,7 +282,7 @@ const arrayPushArgs = valueArgsModel([
 
 
 // $function: arraySet
-// $group: Array
+// $group: array
 // $doc: Set an array element value
 // $arg array: The array
 // $arg index: The index of the element to set
@@ -289,7 +306,7 @@ const arraySetArgs = valueArgsModel([
 
 
 // $function: arrayShift
-// $group: Array
+// $group: array
 // $doc: Remove the first element of the array and return it
 // $arg array: The array
 // $return: The first element of the array; null if the array is empty.
@@ -307,7 +324,7 @@ const arrayShiftArgs = valueArgsModel([
 
 
 // $function: arraySlice
-// $group: Array
+// $group: array
 // $doc: Copy a portion of an array
 // $arg array: The array
 // $arg start: Optional (default is 0). The start index of the slice.
@@ -334,7 +351,7 @@ const arraySliceArgs = valueArgsModel([
 
 
 // $function: arraySort
-// $group: Array
+// $group: array
 // $doc: Sort an array
 // $arg array: The array
 // $arg compareFn: Optional (default is null). The comparison function.
@@ -353,13 +370,67 @@ const arraySortArgs = valueArgsModel([
 ]);
 
 
+//
+// Coverage functions
+//
+
+
+// Coverage configuration object global variable name
+export const coverageGlobalName = '__bareScriptCoverage';
+
+
+// $function: coverageGlobalGet
+// $group: coverage
+// $doc: Get the coverage global object
+// $return: The [coverage global object](https://craigahobbs.github.io/bare-script/model/#var.vName='CoverageGlobal')
+function coverageGlobalGet(unusedArgs, options) {
+    const globals = (options !== null ? (options.globals ?? null) : null);
+    return globals !== null ? (globals[coverageGlobalName] ?? null) : null;
+}
+
+
+// $function: coverageGlobalName
+// $group: coverage
+// $doc: Get the coverage global variable name
+// $return: The coverage global variable name
+function coverageGlobalNameFn() {
+    return coverageGlobalName;
+}
+
+
+// $function: coverageStart
+// $group: coverage
+// $doc: Start coverage data collection
+function coverageStart(unusedArgs, options) {
+    const globals = (options !== null ? (options.globals ?? null) : null);
+    if (globals !== null) {
+        const coverageGlobal = {'enabled': true};
+        globals[coverageGlobalName] = coverageGlobal;
+    }
+}
+
+
+// $function: coverageStop
+// $group: coverage
+// $doc: Stop coverage data collection
+function coverageStop(unusedArgs, options) {
+    const globals = (options !== null ? (options.globals ?? null) : null);
+    if (globals !== null) {
+        const coverageGlobal = globals[coverageGlobalName] ?? null;
+        if (coverageGlobal !== null) {
+            globals[coverageGlobalName].enabled = false;
+        }
+    }
+}
+
+
 //
 // Data functions
 //
 
 
 // $function: dataAggregate
-// $group: Data
+// $group: data
 // $doc: Aggregate a data array
 // $arg data: The data array
 // $arg aggregation: The [aggregation model](https://craigahobbs.github.io/bare-script/library/model.html#var.vName='Aggregation')
@@ -376,7 +447,7 @@ const dataAggregateArgs = valueArgsModel([
 
 
 // $function: dataCalculatedField
-// $group: Data
+// $group: data
 // $doc: Add a calculated field to a data array
 // $arg data: The data array
 // $arg fieldName: The calculated field name
@@ -397,7 +468,7 @@ const dataCalculatedFieldArgs = valueArgsModel([
 
 
 // $function: dataFilter
-// $group: Data
+// $group: data
 // $doc: Filter a data array
 // $arg data: The data array
 // $arg expr: The filter expression
@@ -416,7 +487,7 @@ const dataFilterArgs = valueArgsModel([
 
 
 // $function: dataJoin
-// $group: Data
+// $group: data
 // $doc: Join two data arrays
 // $arg leftData: The left data array
 // $arg rightData: The right data array
@@ -442,7 +513,7 @@ const dataJoinArgs = valueArgsModel([
 
 
 // $function: dataParseCSV
-// $group: Data
+// $group: data
 // $doc: Parse CSV text to a data array
 // $arg text...: The CSV text
 // $return: The data array
@@ -466,7 +537,7 @@ function dataParseCSV(args) {
 
 
 // $function: dataSort
-// $group: Data
+// $group: data
 // $doc: Sort a data array
 // $arg data: The data array
 // $arg sorts: The sort field-name/descending-sort tuples
@@ -483,7 +554,7 @@ const dataSortArgs = valueArgsModel([
 
 
 // $function: dataTop
-// $group: Data
+// $group: data
 // $doc: Keep the top rows for each category
 // $arg data: The data array
 // $arg count: The number of rows to keep (default is 1)
@@ -502,7 +573,7 @@ const dataTopArgs = valueArgsModel([
 
 
 // $function: dataValidate
-// $group: Data
+// $group: data
 // $doc: Validate a data array
 // $arg data: The data array
 // $arg csv: Optional (default is false). If true, parse value strings.
@@ -525,7 +596,7 @@ const dataValidateArgs = valueArgsModel([
 
 
 // $function: datetimeDay
-// $group: Datetime
+// $group: datetime
 // $doc: Get the day of the month of a datetime
 // $arg datetime: The datetime
 // $return: The day of the month
@@ -540,7 +611,7 @@ const datetimeDayArgs = valueArgsModel([
 
 
 // $function: datetimeHour
-// $group: Datetime
+// $group: datetime
 // $doc: Get the hour of a datetime
 // $arg datetime: The datetime
 // $return: The hour
@@ -555,7 +626,7 @@ const datetimeHourArgs = valueArgsModel([
 
 
 // $function: datetimeISOFormat
-// $group: Datetime
+// $group: datetime
 // $doc: Format the datetime as an ISO date/time string
 // $arg datetime: The datetime
 // $arg isDate: If true, format the datetime as an ISO date
@@ -580,7 +651,7 @@ const datetimeISOFormatArgs = valueArgsModel([
 
 
 // $function: datetimeISOParse
-// $group: Datetime
+// $group: datetime
 // $doc: Parse an ISO date/time string
 // $arg string: The ISO date/time string
 // $return: The datetime, or null if parsing fails
@@ -595,7 +666,7 @@ const datetimeISOParseArgs = valueArgsModel([
 
 
 // $function: datetimeMillisecond
-// $group: Datetime
+// $group: datetime
 // $doc: Get the millisecond of a datetime
 // $arg datetime: The datetime
 // $return: The millisecond
@@ -610,7 +681,7 @@ const datetimeMillisecondArgs = valueArgsModel([
 
 
 // $function: datetimeMinute
-// $group: Datetime
+// $group: datetime
 // $doc: Get the minute of a datetime
 // $arg datetime: The datetime
 // $return: The minute
@@ -625,7 +696,7 @@ const datetimeMinuteArgs = valueArgsModel([
 
 
 // $function: datetimeMonth
-// $group: Datetime
+// $group: datetime
 // $doc: Get the month (1-12) of a datetime
 // $arg datetime: The datetime
 // $return: The month
@@ -640,7 +711,7 @@ const datetimeMonthArgs = valueArgsModel([
 
 
 // $function: datetimeNew
-// $group: Datetime
+// $group: datetime
 // $doc: Create a new datetime
 // $arg year: The full year
 // $arg month: The month (1-12)
@@ -667,7 +738,7 @@ const datetimeNewArgs = valueArgsModel([
 
 
 // $function: datetimeNow
-// $group: Datetime
+// $group: datetime
 // $doc: Get the current datetime
 // $return: The current datetime
 function datetimeNow() {
@@ -676,7 +747,7 @@ function datetimeNow() {
 
 
 // $function: datetimeSecond
-// $group: Datetime
+// $group: datetime
 // $doc: Get the second of a datetime
 // $arg datetime: The datetime
 // $return: The second
@@ -691,7 +762,7 @@ const datetimeSecondArgs = valueArgsModel([
 
 
 // $function: datetimeToday
-// $group: Datetime
+// $group: datetime
 // $doc: Get today's datetime
 // $return: Today's datetime
 function datetimeToday() {
@@ -701,7 +772,7 @@ function datetimeToday() {
 
 
 // $function: datetimeYear
-// $group: Datetime
+// $group: datetime
 // $doc: Get the full year of a datetime
 // $arg datetime: The datetime
 // $return: The full year
@@ -721,7 +792,7 @@ const datetimeYearArgs = valueArgsModel([
 
 
 // $function: jsonParse
-// $group: JSON
+// $group: json
 // $doc: Convert a JSON string to an object
 // $arg string: The JSON string
 // $return: The object
@@ -736,7 +807,7 @@ const jsonParseArgs = valueArgsModel([
 
 
 // $function: jsonStringify
-// $group: JSON
+// $group: json
 // $doc: Convert an object to a JSON string
 // $arg value: The object
 // $arg indent: Optional (default is null). The indentation number.
@@ -758,7 +829,7 @@ const jsonStringifyArgs = valueArgsModel([
 
 
 // $function: mathAbs
-// $group: Math
+// $group: math
 // $doc: Compute the absolute value of a number
 // $arg x: The number
 // $return: The absolute value of the number
@@ -773,7 +844,7 @@ const mathAbsArgs = valueArgsModel([
 
 
 // $function: mathAcos
-// $group: Math
+// $group: math
 // $doc: Compute the arccosine, in radians, of a number
 // $arg x: The number
 // $return: The arccosine, in radians, of the number
@@ -788,7 +859,7 @@ const mathAcosArgs = valueArgsModel([
 
 
 // $function: mathAsin
-// $group: Math
+// $group: math
 // $doc: Compute the arcsine, in radians, of a number
 // $arg x: The number
 // $return: The arcsine, in radians, of the number
@@ -803,7 +874,7 @@ const mathAsinArgs = valueArgsModel([
 
 
 // $function: mathAtan
-// $group: Math
+// $group: math
 // $doc: Compute the arctangent, in radians, of a number
 // $arg x: The number
 // $return: The arctangent, in radians, of the number
@@ -818,7 +889,7 @@ const mathAtanArgs = valueArgsModel([
 
 
 // $function: mathAtan2
-// $group: Math
+// $group: math
 // $doc: Compute the angle, in radians, between (0, 0) and a point
 // $arg y: The Y-coordinate of the point
 // $arg x: The X-coordinate of the point
@@ -835,7 +906,7 @@ const mathAtan2Args = valueArgsModel([
 
 
 // $function: mathCeil
-// $group: Math
+// $group: math
 // $doc: Compute the ceiling of a number (round up to the next highest integer)
 // $arg x: The number
 // $return: The ceiling of the number
@@ -850,7 +921,7 @@ const mathCeilArgs = valueArgsModel([
 
 
 // $function: mathCos
-// $group: Math
+// $group: math
 // $doc: Compute the cosine of an angle, in radians
 // $arg x: The angle, in radians
 // $return: The cosine of the angle
@@ -865,7 +936,7 @@ const mathCosArgs = valueArgsModel([
 
 
 // $function: mathFloor
-// $group: Math
+// $group: math
 // $doc: Compute the floor of a number (round down to the next lowest integer)
 // $arg x: The number
 // $return: The floor of the number
@@ -880,7 +951,7 @@ const mathFloorArgs = valueArgsModel([
 
 
 // $function: mathLn
-// $group: Math
+// $group: math
 // $doc: Compute the natural logarithm (base e) of a number
 // $arg x: The number
 // $return: The natural logarithm of the number
@@ -895,7 +966,7 @@ const mathLnArgs = valueArgsModel([
 
 
 // $function: mathLog
-// $group: Math
+// $group: math
 // $doc: Compute the logarithm (base 10) of a number
 // $arg x: The number
 // $arg base: Optional (default is 10). The logarithm base.
@@ -916,7 +987,7 @@ const mathLogArgs = valueArgsModel([
 
 
 // $function: mathMax
-// $group: Math
+// $group: math
 // $doc: Compute the maximum value
 // $arg values...: The values
 // $return: The maximum value
@@ -932,7 +1003,7 @@ function mathMax(values) {
 
 
 // $function: mathMin
-// $group: Math
+// $group: math
 // $doc: Compute the minimum value
 // $arg values...: The values
 // $return: The minimum value
@@ -948,7 +1019,7 @@ function mathMin(values) {
 
 
 // $function: mathPi
-// $group: Math
+// $group: math
 // $doc: Return the number pi
 // $return: The number pi
 function mathPi() {
@@ -957,7 +1028,7 @@ function mathPi() {
 
 
 // $function: mathRandom
-// $group: Math
+// $group: math
 // $doc: Compute a random number between 0 and 1, inclusive
 // $return: A random number
 function mathRandom() {
@@ -966,7 +1037,7 @@ function mathRandom() {
 
 
 // $function: mathRound
-// $group: Math
+// $group: math
 // $doc: Round a number to a certain number of decimal places
 // $arg x: The number
 // $arg digits: Optional (default is 0). The number of decimal digits to round to.
@@ -983,7 +1054,7 @@ const mathRoundArgs = valueArgsModel([
 
 
 // $function: mathSign
-// $group: Math
+// $group: math
 // $doc: Compute the sign of a number
 // $arg x: The number
 // $return: -1 for a negative number, 1 for a positive number, and 0 for zero
@@ -998,7 +1069,7 @@ const mathSignArgs = valueArgsModel([
 
 
 // $function: mathSin
-// $group: Math
+// $group: math
 // $doc: Compute the sine of an angle, in radians
 // $arg x: The angle, in radians
 // $return: The sine of the angle
@@ -1013,7 +1084,7 @@ const mathSinArgs = valueArgsModel([
 
 
 // $function: mathSqrt
-// $group: Math
+// $group: math
 // $doc: Compute the square root of a number
 // $arg x: The number
 // $return: The square root of the number
@@ -1028,7 +1099,7 @@ const mathSqrtArgs = valueArgsModel([
 
 
 // $function: mathTan
-// $group: Math
+// $group: math
 // $doc: Compute the tangent of an angle, in radians
 // $arg x: The angle, in radians
 // $return: The tangent of the angle
@@ -1048,7 +1119,7 @@ const mathTanArgs = valueArgsModel([
 
 
 // $function: numberParseFloat
-// $group: Number
+// $group: number
 // $doc: Parse a string as a floating point number
 // $arg string: The string
 // $return: The number
@@ -1063,7 +1134,7 @@ const numberParseFloatArgs = valueArgsModel([
 
 
 // $function: numberParseInt
-// $group: Number
+// $group: number
 // $doc: Parse a string as an integer
 // $arg string: The string
 // $arg radix: Optional (default is 10). The number base.
@@ -1080,7 +1151,7 @@ const numberParseIntArgs = valueArgsModel([
 
 
 // $function: numberToFixed
-// $group: Number
+// $group: number
 // $doc: Format a number using fixed-point notation
 // $arg x: The number
 // $arg digits: Optional (default is 2). The number of digits to appear after the decimal point.
@@ -1110,7 +1181,7 @@ const rNumberCleanup = /\.0*$/;
 
 
 // $function: objectAssign
-// $group: Object
+// $group: object
 // $doc: Assign the keys/values of one object to another
 // $arg object: The object to assign to
 // $arg object2: The object to assign
@@ -1128,7 +1199,7 @@ const objectAssignArgs = valueArgsModel([
 
 
 // $function: objectCopy
-// $group: Object
+// $group: object
 // $doc: Create a copy of an object
 // $arg object: The object to copy
 // $return: The object copy
@@ -1143,7 +1214,7 @@ const objectCopyArgs = valueArgsModel([
 
 
 // $function: objectDelete
-// $group: Object
+// $group: object
 // $doc: Delete an object key
 // $arg object: The object
 // $arg key: The key to delete
@@ -1159,7 +1230,7 @@ const objectDeleteArgs = valueArgsModel([
 
 
 // $function: objectGet
-// $group: Object
+// $group: object
 // $doc: Get an object key's value
 // $arg object: The object
 // $arg key: The key
@@ -1179,7 +1250,7 @@ const objectGetArgs = valueArgsModel([
 
 
 // $function: objectHas
-// $group: Object
+// $group: object
 // $doc: Test if an object contains a key
 // $arg object: The object
 // $arg key: The key
@@ -1196,7 +1267,7 @@ const objectHasArgs = valueArgsModel([
 
 
 // $function: objectKeys
-// $group: Object
+// $group: object
 // $doc: Get an object's keys
 // $arg object: The object
 // $return: The array of keys
@@ -1211,7 +1282,7 @@ const objectKeysArgs = valueArgsModel([
 
 
 // $function: objectNew
-// $group: Object
+// $group: object
 // $doc: Create a new object
 // $arg keyValues...: The object's initial key and value pairs
 // $return: The new object
@@ -1230,7 +1301,7 @@ function objectNew(keyValues) {
 
 
 // $function: objectSet
-// $group: Object
+// $group: object
 // $doc: Set an object key's value
 // $arg object: The object
 // $arg key: The key
@@ -1255,7 +1326,7 @@ const objectSetArgs = valueArgsModel([
 
 
 // $function: regexEscape
-// $group: Regex
+// $group: regex
 // $doc: Escape a string for use in a regular expression
 // $arg string: The string to escape
 // $return: The escaped string
@@ -1272,7 +1343,7 @@ const rRegexEscape = /[.*+?^${}()|[\]\\]/g;
 
 
 // $function: regexMatch
-// $group: Regex
+// $group: regex
 // $doc: Find the first match of a regular expression in a string
 // $arg regex: The regular expression
 // $arg string: The string
@@ -1291,7 +1362,7 @@ const regexMatchArgs = valueArgsModel([
 
 
 // $function: regexMatchAll
-// $group: Regex
+// $group: regex
 // $doc: Find all matches of regular expression in a string
 // $arg regex: The regular expression
 // $arg string: The string
@@ -1374,7 +1445,7 @@ struct RegexMatch
 
 
 // $function: regexNew
-// $group: Regex
+// $group: regex
 // $doc: Create a regular expression
 // eslint-disable-next-line max-len
 // $arg pattern: The [regular expression pattern string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_expressions#writing_a_regular_expression_pattern)
@@ -1405,7 +1476,7 @@ const regexNewArgs = valueArgsModel([
 
 
 // $function: regexReplace
-// $group: Regex
+// $group: regex
 // $doc: Replace regular expression matches with a string
 // $arg regex: The replacement regular expression
 // $arg string: The string
@@ -1425,7 +1496,7 @@ const regexReplaceArgs = valueArgsModel([
 
 
 // $function: regexSplit
-// $group: Regex
+// $group: regex
 // $doc: Split a string with a regular expression
 // $arg regex: The regular expression
 // $arg string: The string
@@ -1447,7 +1518,7 @@ const regexSplitArgs = valueArgsModel([
 
 
 // $function: schemaParse
-// $group: Schema
+// $group: schema
 // $doc: Parse the [Schema Markdown](https://craigahobbs.github.io/schema-markdown-js/language/) text
 // $arg lines...: The [Schema Markdown](https://craigahobbs.github.io/schema-markdown-js/language/)
 // $arg lines...: text lines (may contain nested arrays of un-split lines)
@@ -1458,7 +1529,7 @@ function schemaParse(lines) {
 
 
 // $function: schemaParseEx
-// $group: Schema
+// $group: schema
 // $doc: Parse the [Schema Markdown](https://craigahobbs.github.io/schema-markdown-js/language/) text with options
 // $arg lines: The array of [Schema Markdown](https://craigahobbs.github.io/schema-markdown-js/language/)
 // $arg lines: text lines (may contain nested arrays of un-split lines)
@@ -1484,7 +1555,7 @@ const schemaParseExArgs = valueArgsModel([
 
 
 // $function: schemaTypeModel
-// $group: Schema
+// $group: schema
 // $doc: Get the [Schema Markdown Type Model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types')
 // $return: The [Schema Markdown Type Model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types')
 function schemaTypeModel() {
@@ -1493,7 +1564,7 @@ function schemaTypeModel() {
 
 
 // $function: schemaValidate
-// $group: Schema
+// $group: schema
 // $doc: Validate an object to a schema type
 // $arg types: The [type model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types')
 // $arg typeName: The type name
@@ -1513,7 +1584,7 @@ const schemaValidateArgs = valueArgsModel([
 
 
 // $function: schemaValidateTypeModel
-// $group: Schema
+// $group: schema
 // $doc: Validate a [Schema Markdown Type Model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types')
 // $arg types: The [type model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types') to validate
 // $return: The validated [type model](https://craigahobbs.github.io/schema-markdown-doc/doc/#var.vName='Types')
@@ -1533,7 +1604,7 @@ const schemaValidateTypeModelArgs = valueArgsModel([
 
 
 // $function: stringCharCodeAt
-// $group: String
+// $group: string
 // $doc: Get a string index's character code
 // $arg string: The string
 // $arg index: The character index
@@ -1554,7 +1625,7 @@ const stringCharCodeAtArgs = valueArgsModel([
 
 
 // $function: stringEndsWith
-// $group: String
+// $group: string
 // $doc: Determine if a string ends with a search string
 // $arg string: The string
 // $arg search: The search string
@@ -1571,7 +1642,7 @@ const stringEndsWithArgs = valueArgsModel([
 
 
 // $function: stringFromCharCode
-// $group: String
+// $group: string
 // $doc: Create a string of characters from character codes
 // $arg charCodes...: The character codes
 // $return: The string of characters
@@ -1587,7 +1658,7 @@ function stringFromCharCode(charCodes) {
 
 
 // $function: stringIndexOf
-// $group: String
+// $group: string
 // $doc: Find the first index of a search string in a string
 // $arg string: The string
 // $arg search: The search string
@@ -1610,7 +1681,7 @@ const stringIndexOfArgs = valueArgsModel([
 
 
 // $function: stringLastIndexOf
-// $group: String
+// $group: string
 // $doc: Find the last index of a search string in a string
 // $arg string: The string
 // $arg search: The search string
@@ -1634,7 +1705,7 @@ const stringLastIndexOfArgs = valueArgsModel([
 
 
 // $function: stringLength
-// $group: String
+// $group: string
 // $doc: Get the length of a string
 // $arg string: The string
 // $return: The string's length; zero if not a string
@@ -1649,7 +1720,7 @@ const stringLengthArgs = valueArgsModel([
 
 
 // $function: stringLower
-// $group: String
+// $group: string
 // $doc: Convert a string to lower-case
 // $arg string: The string
 // $return: The lower-case string
@@ -1664,7 +1735,7 @@ const stringLowerArgs = valueArgsModel([
 
 
 // $function: stringNew
-// $group: String
+// $group: string
 // $doc: Create a new string from a value
 // $arg value: The value
 // $return: The new string
@@ -1674,7 +1745,7 @@ function stringNew([value = null]) {
 
 
 // $function: stringRepeat
-// $group: String
+// $group: string
 // $doc: Repeat a string
 // $arg string: The string to repeat
 // $arg count: The number of times to repeat the string
@@ -1691,7 +1762,7 @@ const stringRepeatArgs = valueArgsModel([
 
 
 // $function: stringReplace
-// $group: String
+// $group: string
 // $doc: Replace all instances of a string with another string
 // $arg string: The string to update
 // $arg substr: The string to replace
@@ -1710,7 +1781,7 @@ const stringReplaceArgs = valueArgsModel([
 
 
 // $function: stringSlice
-// $group: String
+// $group: string
 // $doc: Copy a portion of a string
 // $arg string: The string
 // $arg start: The start index of the slice
@@ -1737,7 +1808,7 @@ const stringSliceArgs = valueArgsModel([
 
 
 // $function: stringSplit
-// $group: String
+// $group: string
 // $doc: Split a string
 // $arg string: The string to split
 // $arg separator: The separator string
@@ -1754,7 +1825,7 @@ const stringSplitArgs = valueArgsModel([
 
 
 // $function: stringStartsWith
-// $group: String
+// $group: string
 // $doc: Determine if a string starts with a search string
 // $arg string: The string
 // $arg search: The search string
@@ -1771,7 +1842,7 @@ const stringStartsWithArgs = valueArgsModel([
 
 
 // $function: stringTrim
-// $group: String
+// $group: string
 // $doc: Trim the whitespace from the beginning and end of a string
 // $arg string: The string
 // $return: The trimmed string
@@ -1786,7 +1857,7 @@ const stringTrimArgs = valueArgsModel([
 
 
 // $function: stringUpper
-// $group: String
+// $group: string
 // $doc: Convert a string to upper-case
 // $arg string: The string
 // $return: The upper-case string
@@ -1806,7 +1877,7 @@ const stringUpperArgs = valueArgsModel([
 
 
 // $function: systemBoolean
-// $group: System
+// $group: system
 // $doc: Interpret a value as a boolean
 // $arg value: The value
 // $return: true or false
@@ -1816,7 +1887,7 @@ function systemBoolean([value = null]) {
 
 
 // $function: systemCompare
-// $group: System
+// $group: system
 // $doc: Compare two values
 // $arg left: The left value
 // $arg right: The right value
@@ -1827,7 +1898,7 @@ function systemCompare([left = null, right = null]) {
 
 
 // $function: systemFetch
-// $group: System
+// $group: system
 // $doc: Retrieve a URL resource
 // $arg url: The resource URL,
 // $arg url: [request model](https://craigahobbs.github.io/bare-script/library/model.html#var.vName='SystemFetchRequest'),
@@ -1918,8 +1989,31 @@ struct SystemFetchRequest
 `);
 
 
+// System includes object global variable name
+export const systemGlobalIncludesName = '__bareScriptIncludes';
+
+
+// $function: systemGlobalIncludesGet
+// $group: system
+// $doc: Get the global system includes object
+// $return: The global system includes object
+function systemGlobalIncludesGet(unusedArgs, options) {
+    const globals = (options !== null ? (options.globals ?? null) : null);
+    return (globals !== null ? (globals[systemGlobalIncludesName] ?? null) : null);
+}
+
+
+// $function: systemGlobalIncludesName
+// $group: system
+// $doc: Get the system includes object global variable name
+// $return: The system includes object global variable name
+function systemGlobalIncludesNameFn() {
+    return systemGlobalIncludesName;
+}
+
+
 // $function: systemGlobalGet
-// $group: System
+// $group: system
 // $doc: Get a global variable value
 // $arg name: The global variable name
 // $arg defaultValue: The default value (optional)
@@ -1937,7 +2031,7 @@ const systemGlobalGetArgs = valueArgsModel([
 
 
 // $function: systemGlobalSet
-// $group: System
+// $group: system
 // $doc: Set a global variable value
 // $arg name: The global variable name
 // $arg value: The global variable's value
@@ -1958,7 +2052,7 @@ const systemGlobalSetArgs = valueArgsModel([
 
 
 // $function: systemIs
-// $group: System
+// $group: system
 // $doc: Test if one value is the same object as another
 // $arg value1: The first value
 // $arg value2: The second value
@@ -1969,7 +2063,7 @@ function systemIs([value1 = null, value2 = null]) {
 
 
 // $function: systemLog
-// $group: System
+// $group: system
 // $doc: Log a message to the console
 // $arg message: The log message
 function systemLog([message = null], options) {
@@ -1980,7 +2074,7 @@ function systemLog([message = null], options) {
 
 
 // $function: systemLogDebug
-// $group: System
+// $group: system
 // $doc: Log a message to the console, if in debug mode
 // $arg message: The log message
 function systemLogDebug([message = null], options) {
@@ -1991,7 +2085,7 @@ function systemLogDebug([message = null], options) {
 
 
 // $function: systemPartial
-// $group: System
+// $group: system
 // $doc: Return a new function which behaves like "func" called with "args".
 // $doc: If additional arguments are passed to the returned function, they are appended to "args".
 // $arg func: The function
@@ -2017,7 +2111,7 @@ const systemPartialArgs = valueArgsModel([
 
 
 // $function: systemType
-// $group: System
+// $group: system
 // $doc: Get a value's type string
 // $arg value: The value
 // $return: The type string of the value.
@@ -2033,7 +2127,7 @@ function systemType([value = null]) {
 
 
 // $function: urlEncode
-// $group: URL
+// $group: url
 // $doc: Encode a URL
 // $arg url: The URL string
 // $return: The encoded URL string
@@ -2054,7 +2148,7 @@ const urlEncodeArgs = valueArgsModel([
 
 
 // $function: urlEncodeComponent
-// $group: URL
+// $group: url
 // $doc: Encode a URL component
 // $arg url: The URL component string
 // $return: The encoded URL component string
@@ -2079,6 +2173,7 @@ export const scriptFunctions = {
     arrayCopy,
     arrayDelete,
     arrayExtend,
+    arrayFlat,
     arrayGet,
     arrayIndexOf,
     arrayJoin,
@@ -2092,6 +2187,10 @@ export const scriptFunctions = {
     arrayShift,
     arraySlice,
     arraySort,
+    coverageGlobalGet,
+    'coverageGlobalName': coverageGlobalNameFn,
+    coverageStart,
+    coverageStop,
     dataAggregate,
     dataCalculatedField,
     dataFilter,
@@ -2174,6 +2273,8 @@ export const scriptFunctions = {
     systemCompare,
     systemFetch,
     systemGlobalGet,
+    systemGlobalIncludesGet,
+    'systemGlobalIncludesName': systemGlobalIncludesNameFn,
     systemGlobalSet,
     systemIs,
     systemLog,
@@ -2189,6 +2290,7 @@ export const scriptFunctions = {
 export const expressionFunctionMap = {
     'abs': 'mathAbs',
     'acos': 'mathAcos',
+    'arrayNew': 'arrayNew',
     'asin': 'mathAsin',
     'atan': 'mathAtan',
     'atan2': 'mathAtan2',
@@ -2213,6 +2315,7 @@ export const expressionFunctionMap = {
     'minute': 'datetimeMinute',
     'month': 'datetimeMonth',
     'now': 'datetimeNow',
+    'objectNew': 'objectNew',
     'parseInt': 'numberParseInt',
     'parseFloat': 'numberParseFloat',
     'pi': 'mathPi',
