@@ -17,6 +17,15 @@ struct BareScript
     # The script's statements
     ScriptStatement[] statements
 
+    # The script name
+    optional string scriptName
+
+    # The script's lines
+    optional string[] scriptLines
+
+    # If true, this is a system include script
+    optional bool system
+
 
 # A script statement
 union ScriptStatement
@@ -31,7 +40,7 @@ union ScriptStatement
     ReturnStatement return
 
     # A label definition
-    string label
+    LabelStatement label
 
     # A function definition
     FunctionStatement function
@@ -40,8 +49,18 @@ union ScriptStatement
     IncludeStatement include
 
 
+# Script statement base struct
+struct BaseStatement
+
+    # The script statement's line number
+    optional int lineNumber
+
+    # The number of lines in the script statement (default is 1)
+    optional int lineCount
+
+
 # An expression statement
-struct ExpressionStatement
+struct ExpressionStatement (BaseStatement)
 
     # The variable name to assign the expression value
     optional string name
@@ -51,7 +70,7 @@ struct ExpressionStatement
 
 
 # A jump statement
-struct JumpStatement
+struct JumpStatement (BaseStatement)
 
     # The label to jump to
     string label
@@ -61,14 +80,21 @@ struct JumpStatement
 
 
 # A return statement
-struct ReturnStatement
+struct ReturnStatement (BaseStatement)
 
     # The expression to return
     optional Expression expr
 
 
+# A label statement
+struct LabelStatement (BaseStatement)
+
+    # The label name
+    string name
+
+
 # A function definition statement
-struct FunctionStatement
+struct FunctionStatement (BaseStatement)
 
     # If true, the function is defined as async
     optional bool async
@@ -87,7 +113,7 @@ struct FunctionStatement
 
 
 # An include statement
-struct IncludeStatement
+struct IncludeStatement (BaseStatement)
 
     # The list of include scripts to load and execute in the global scope
     IncludeScript[len > 0] includes
@@ -103,6 +129,36 @@ struct IncludeScript
     optional bool system
 
 
+# The coverage global configuration
+struct CoverageGlobal
+
+    # If true, coverage is enabled
+    optional bool enabled
+
+    # The map of script name to script coverage
+    optional CoverageGlobalScript{} scripts
+
+
+# The script coverage
+struct CoverageGlobalScript
+
+    # The script
+    BareScript script
+
+    # The map of script line number string to script statement coverage
+    CoverageGlobalStatement{} covered
+
+
+# The script statement coverage
+struct CoverageGlobalStatement
+
+    # The script statement
+    ScriptStatement statement
+
+    # The statement's coverage count
+    int count
+
+
 # An expression
 union Expression
 
@@ -264,25 +320,34 @@ export function validateExpression(expr) {
  * Lint a BareScript script model
  *
  * @param {Object} script - The [BareScript model](./model/#var.vName='BareScript')
+ * @param {?Object} globals - The script global variables
  * @returns {string[]} The array of lint warnings
  */
-export function lintScript(script) {
+export function lintScript(script, globals = null) {
     const warnings = [];
+    const {statements} = script;
 
     // Empty script?
-    if (script.statements.length === 0) {
-        warnings.push('Empty script');
+    if (statements.length === 0) {
+        lintScriptWarning(warnings, script, null, 'Empty script');
     }
 
     // Variable used before assignment?
     const varAssigns = {};
     const varUses = {};
-    getVariableAssignmentsAndUses(script.statements, varAssigns, varUses);
+    getVariableAssignmentsAndUses(statements, varAssigns, varUses);
     for (const varName of Object.keys(varAssigns)) {
         if (varName in varUses && varUses[varName] <= varAssigns[varName]) {
-            warnings.push(
-                `Global variable "${varName}" used (index ${varUses[varName]}) before assignment (index ${varAssigns[varName]})`
-            );
+            lintScriptWarning(warnings, script, statements[varUses[varName]], `Global variable "${varName}" used before assignment`);
+        }
+    }
+
+    // Unknown global variable?
+    if (globals !== null) {
+        for (const varName of Object.keys(varUses).sort()) {
+            if (!(varName in varAssigns) && !(varName in globals) && !builtinGlobals.has(varName)) {
+                lintScriptWarning(warnings, script, statements[varUses[varName]], `Unknown global variable "${varName}"`);
+            }
         }
     }
 
@@ -290,14 +355,14 @@ export function lintScript(script) {
     const functionsDefined = {};
     const labelsDefined = {};
     const labelsUsed = {};
-    for (const [ixStatement, statement] of script.statements.entries()) {
+    for (const [ixStatement, statement] of statements.entries()) {
         const [statementKey] = Object.keys(statement);
 
         // Function definition checks
         if (statementKey === 'function') {
             // Function redefinition?
             if (statement.function.name in functionsDefined) {
-                warnings.push(`Redefinition of function "${statement.function.name}" (index ${ixStatement})`);
+                lintScriptWarning(warnings, script, statement, `Redefinition of function "${statement.function.name}"`);
             } else {
                 functionsDefined[statement.function.name] = ixStatement;
             }
@@ -306,16 +371,17 @@ export function lintScript(script) {
             const fnVarAssigns = {};
             const fnVarUses = {};
             const args = (statement.function.args ?? null);
-            getVariableAssignmentsAndUses(statement.function.statements, fnVarAssigns, fnVarUses);
+            const fnStatements = statement.function.statements;
+            getVariableAssignmentsAndUses(fnStatements, fnVarAssigns, fnVarUses);
             for (const varName of Object.keys(fnVarAssigns)) {
                 // Ignore re-assigned function arguments
                 if (args !== null && args.indexOf(varName) !== -1) {
                     continue;
                 }
                 if (varName in fnVarUses && fnVarUses[varName] <= fnVarAssigns[varName]) {
-                    warnings.push(
-                        `Variable "${varName}" of function "${statement.function.name}" used (index ${fnVarUses[varName]}) ` +
-                            `before assignment (index ${fnVarAssigns[varName]})`
+                    lintScriptWarning(
+                        warnings, script, fnStatements[fnVarUses[varName]],
+                        `Variable "${varName}" of function "${statement.function.name}" used before assignment`
                     );
                 }
             }
@@ -323,25 +389,42 @@ export function lintScript(script) {
             // Unused variables?
             for (const varName of Object.keys(fnVarAssigns)) {
                 if (!(varName in fnVarUses)) {
-                    warnings.push(
-                        `Unused variable "${varName}" defined in function "${statement.function.name}" (index ${fnVarAssigns[varName]})`
+                    lintScriptWarning(
+                        warnings, script, fnStatements[fnVarAssigns[varName]],
+                        `Unused variable "${varName}" defined in function "${statement.function.name}"`
                     );
                 }
             }
 
+            // Unknown global variable?
+            if (globals !== null) {
+                for (const varName of Object.keys(fnVarUses).sort()) {
+                    if (!(varName in fnVarAssigns) && (args === null || args.indexOf(varName) === -1) &&
+                        !(varName in globals) && !builtinGlobals.has(varName)) {
+                        lintScriptWarning(
+                            warnings, script, fnStatements[fnVarUses[varName]], `Unknown global variable "${varName}"`
+                        );
+                    }
+                }
+            }
+
             // Function argument checks
             if (args !== null) {
                 const argsDefined = new Set();
                 for (const arg of args) {
                     // Duplicate argument?
                     if (argsDefined.has(arg)) {
-                        warnings.push(`Duplicate argument "${arg}" of function "${statement.function.name}" (index ${ixStatement})`);
+                        lintScriptWarning(
+                            warnings, script, statement, `Duplicate argument "${arg}" of function "${statement.function.name}"`
+                        );
                     } else {
                         argsDefined.add(arg);
 
                         // Unused argument?
                         if (!(arg in fnVarUses)) {
-                            warnings.push(`Unused argument "${arg}" of function "${statement.function.name}" (index ${ixStatement})`);
+                            lintScriptWarning(
+                                warnings, script, statement, `Unused argument "${arg}" of function "${statement.function.name}"`
+                            );
                         }
                     }
                 }
@@ -350,25 +433,33 @@ export function lintScript(script) {
             // Iterate function statements
             const fnLabelsDefined = {};
             const fnLabelsUsed = {};
-            for (const [ixFnStatement, fnStatement] of statement.function.statements.entries()) {
+            let hasAsyncStatement = false;
+            for (const [ixFnStatement, fnStatement] of fnStatements.entries()) {
                 const [fnStatementKey] = Object.keys(fnStatement);
 
+                // Any async statements?
+                if (globals !== null) {
+                    hasAsyncStatement ||= isAsyncStatement(fnStatement, globals, statement.function.async ?? false);
+                }
+
                 // Function expression statement checks
                 if (fnStatementKey === 'expr') {
                     // Pointless function expression statement?
                     if (!('name' in fnStatement.expr) && isPointlessExpression(fnStatement.expr.expr)) {
-                        warnings.push(`Pointless statement in function "${statement.function.name}" (index ${ixFnStatement})`);
+                        lintScriptWarning(warnings, script, statement, `Pointless statement in function "${statement.function.name}"`);
                     }
 
                 // Function label statement checks
                 } else if (fnStatementKey === 'label') {
                     // Label redefinition?
-                    if (fnStatement.label in fnLabelsDefined) {
-                        warnings.push(
-                            `Redefinition of label "${fnStatement.label}" in function "${statement.function.name}" (index ${ixFnStatement})`
+                    const fnStatementLabel = fnStatement.label.name;
+                    if (fnStatementLabel in fnLabelsDefined) {
+                        lintScriptWarning(
+                            warnings, script, statement,
+                            `Redefinition of label "${fnStatementLabel}" in function "${statement.function.name}"`
                         );
                     } else {
-                        fnLabelsDefined[fnStatement.label] = ixFnStatement;
+                        fnLabelsDefined[fnStatementLabel] = ixFnStatement;
                     }
 
                 // Function jump statement checks
@@ -382,14 +473,23 @@ export function lintScript(script) {
             // Unused function labels?
             for (const label of Object.keys(fnLabelsDefined)) {
                 if (!(label in fnLabelsUsed)) {
-                    warnings.push(`Unused label "${label}" in function "${statement.function.name}" (index ${fnLabelsDefined[label]})`);
+                    lintScriptWarning(warnings, script, statement, `Unused label "${label}" in function "${statement.function.name}"`);
                 }
             }
 
             // Unknown function labels?
             for (const label of Object.keys(fnLabelsUsed)) {
                 if (!(label in fnLabelsDefined)) {
-                    warnings.push(`Unknown label "${label}" in function "${statement.function.name}" (index ${fnLabelsUsed[label]})`);
+                    lintScriptWarning(warnings, script, statement, `Unknown label "${label}" in function "${statement.function.name}"`);
+                }
+            }
+
+            // Async function issues?
+            if (globals !== null) {
+                if (statement.function.async && !hasAsyncStatement) {
+                    lintScriptWarning(warnings, script, statement, `Unecessary async function "${statement.function.name}"`);
+                } else if (!statement.function.async && hasAsyncStatement) {
+                    lintScriptWarning(warnings, script, statement, `Function "${statement.function.name}" requires async`);
                 }
             }
 
@@ -397,16 +497,17 @@ export function lintScript(script) {
         } else if (statementKey === 'expr') {
             // Pointless global expression statement?
             if (!('name' in statement.expr) && isPointlessExpression(statement.expr.expr)) {
-                warnings.push(`Pointless global statement (index ${ixStatement})`);
+                lintScriptWarning(warnings, script, statement, 'Pointless global statement');
             }
 
         // Global label statement checks
         } else if (statementKey === 'label') {
             // Label redefinition?
-            if (statement.label in labelsDefined) {
-                warnings.push(`Redefinition of global label "${statement.label}" (index ${ixStatement})`);
+            const statementLabel = statement.label.name;
+            if (statementLabel in labelsDefined) {
+                lintScriptWarning(warnings, script, statement, `Redefinition of global label "${statementLabel}"`);
             } else {
-                labelsDefined[statement.label] = ixStatement;
+                labelsDefined[statementLabel] = ixStatement;
             }
 
         // Global jump statement checks
@@ -420,14 +521,14 @@ export function lintScript(script) {
     // Unused global labels?
     for (const label of Object.keys(labelsDefined)) {
         if (!(label in labelsUsed)) {
-            warnings.push(`Unused global label "${label}" (index ${labelsDefined[label]})`);
+            lintScriptWarning(warnings, script, statements[labelsDefined[label]], `Unused global label "${label}"`);
         }
     }
 
     // Unknown global labels?
     for (const label of Object.keys(labelsUsed)) {
         if (!(label in labelsDefined)) {
-            warnings.push(`Unknown global label "${label}" (index ${labelsUsed[label]})`);
+            lintScriptWarning(warnings, script, statements[labelsUsed[label]], `Unknown global label "${label}"`);
         }
     }
 
@@ -435,6 +536,18 @@ export function lintScript(script) {
 }
 
 
+// Builtin global variable names
+const builtinGlobals = new Set(['false', 'if', 'null', 'true']);
+
+
+// Helper to format static analysis warnings
+function lintScriptWarning(warnings, script, statement, message) {
+    const scriptName = script.scriptName ?? '';
+    const lineno = (statement !== null ? (statement[Object.keys(statement)[0]].lineNumber ?? 1) : 1);
+    warnings.push(`${scriptName}:${lineno}: ${message}`);
+}
+
+
 // Helper function to determine if an expression statement's expression is pointless
 function isPointlessExpression(expr) {
     const [exprKey] = Object.keys(expr);
@@ -451,6 +564,43 @@ function isPointlessExpression(expr) {
 }
 
 
+// Helper function to determine if a statement requires async
+function isAsyncStatement(statement, globals, isAsync) {
+    const [statementKey] = Object.keys(statement);
+    if (statementKey === 'expr') {
+        return isAsyncExpression(statement.expr.expr, globals, isAsync);
+    } else if (statementKey === 'jump') {
+        return 'expr' in statement.jump ? isAsyncExpression(statement.jump.expr, globals, isAsync) : false;
+    } else if (statementKey === 'return') {
+        return 'expr' in statement.return ? isAsyncExpression(statement.return.expr, globals, isAsync) : false;
+    }
+    return false;
+}
+
+
+// Helper function to determine if an expression statement requires async
+function isAsyncExpression(expr, globals, isAsync) {
+    const [exprKey] = Object.keys(expr);
+    if (exprKey === 'function') {
+        // Global function not exist? Assume its OK for the scope
+        const funcValue = globals[expr.function.name] ?? null;
+        if (funcValue === null) {
+            return isAsync;
+        }
+
+        // Is function async?
+        return typeof funcValue === 'function' && funcValue.constructor.name === 'AsyncFunction';
+    } else if (exprKey === 'binary') {
+        return isAsyncExpression(expr.binary.left, globals, isAsync) || isAsyncExpression(expr.binary.right, globals, isAsync);
+    } else if (exprKey === 'unary') {
+        return isAsyncExpression(expr.unary.expr, globals, isAsync);
+    } else if (exprKey === 'group') {
+        return isAsyncExpression(expr.group, globals, isAsync);
+    }
+    return false;
+}
+
+
 // Helper function to set variable assignments/uses for a statements array
 function getVariableAssignmentsAndUses(statements, assigns, uses) {
     for (const [ixStatement, statement] of statements.entries()) {
