@@ -5,10 +5,10 @@
 Chisel action class
 """
 
-from cgi import parse_header # pylint: disable=deprecated-module
 from functools import partial
 from http import HTTPStatus
 from json import loads as json_loads
+import re
 
 from schema_markdown import ValidationError, decode_query_string, get_referenced_types, parse_schema_markdown, validate_type
 
@@ -16,6 +16,10 @@ from .app import Context
 from .request import Request
 
 
+# Regex for parsing the Content-Type header
+RE_CONTENT_TYPE_HEADER = re.compile(r'\bcharset\s*=\s*(?P<charset>\S+)')
+
+
 def action(action_callback=None, **kwargs):
     """
     Decorator for creating an :class:`~chisel.Action` object that wraps an action callback function. For example:
@@ -240,7 +244,8 @@ class Action(Request):
             try:
                 if content:
                     content_type = environ.get('CONTENT_TYPE')
-                    content_charset = ('utf-8' if content_type is None else parse_header(content_type)[1].get('charset', 'utf-8'))
+                    match_charset = None if content_type is None else RE_CONTENT_TYPE_HEADER.search(content_type)
+                    content_charset = 'utf-8' if match_charset is None else match_charset.group('charset')
                     content_json = content.decode(content_charset)
                     request = json_loads(content_json)
                 else:
@@ -332,7 +337,7 @@ class Action(Request):
                 raise _ActionErrorInternal(HTTPStatus.INTERNAL_SERVER_ERROR, 'UnexpectedError')
 
             # Validate the response
-            if validate_output and ctx.app.validate_output:
+            if not self.wsgi_response and validate_output and ctx.app.validate_output:
                 try:
                     validate_type(output_types, output_type, response)
                 except ValidationError as exc:
